//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.Text.RegularExpressions;
using System.Xml;

using MindFusion.Diagramming.Components;


namespace MindFusion.Diagramming.WinForms.Samples.CS.Scripting
{
	public class FunctionNode : ScriptedNode
	{
		private static readonly string ContentTemplate = @"
			<Border Padding=""2,3,2,2"" Background=""Orange"">
				<GridPanel>
					<GridPanel.Rows>
						<GridRow Height=""Auto"" />
						<GridRow Height=""1"" />
						<GridRow Height=""Auto"" />
						<GridRow Height=""1"" />
						<GridRow />
					</GridPanel.Rows>
					<GridPanel.Columns>
						<GridColumn />
					</GridPanel.Columns>

					<Text Text=""Function"" Font=""Verdana, 3world, style=Bold"" />
					<Edit Name=""Title"" Text=""Name"" Font=""Verdana, 3world, style=Bold"" GridRow=""2"" TextChanged=""OnTextChanged"" />
					<Edit Name=""Code"" Text="""" Font=""Verdana, 3world"" GridRow=""4"" TextChanged=""OnTextChanged"" />

				</GridPanel>
			</Border>";

		public FunctionNode(Diagram diagram)
			: base(diagram)
		{
			AllowIncomingLinks = false;
			AllowOutgoingLinks = false;
		}

		protected override void SaveToXml(XmlElement xmlElement, XmlPersistContext context)
		{
			base.SaveToXml(xmlElement, context);

			// Save title & code
			context.WriteString(Title, "Title", xmlElement);
			context.WriteString(Code, "Code", xmlElement);
		}

		protected override void LoadFromXml(XmlElement xmlElement, XmlPersistContext context)
		{
			base.LoadFromXml(xmlElement, context);

			// Load name & code
			Title = context.ReadString("Title", xmlElement);
			Code = context.ReadString("Code", xmlElement);
		}

		protected override ComponentBase LoadContent()
		{
			return XmlLoader.Load(ContentTemplate, this);
		}

		protected override void OnLoaded()
		{
			base.OnLoaded();

			title = FindComponent("Title") as EditComponent;
			code = FindComponent("Code") as EditComponent;
		}

		private void OnTextChanged(object sender, TextEventArgs e)
		{
			EnsureSize();
		}


		public string Title
		{
			get { return title.Text; }
			set { title.Text = value; }
		}

		public string Name
		{
			get
			{
				string title = Title.Trim();
				int bracket = title.IndexOf('(');
				if (bracket == -1)
					return title;

				return title.Substring(0, bracket).Trim();
			}
		}

		public string[] ParameterNames
		{
			get
			{
				// Add the function arguments
				string title = Title;
				Regex regex = new Regex(@"\(.*\)");
				MatchCollection matches = regex.Matches(title);
				if (matches.Count > 0)
				{
					string parameters = matches[0].Value;
					parameters = parameters.Substring(1, parameters.Length - 2).Trim();
					return parameters.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);
				}

				return null;
			}
		}

		public string Code
		{
			get { return code.Text; }
			set { code.Text = value; }
		}


		private EditComponent title;
		private EditComponent code;
	}
}
