//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.Xml;

using MindFusion.Diagramming.Components;


namespace MindFusion.Diagramming.WinForms.Samples.CS.Scripting
{
	public class OperationNode : ScriptedNode
	{
		private static readonly string ContentTemplate = @"
			<Border Padding=""2,3,2,2"">
				<GridPanel>
					<GridPanel.Rows>
						<GridRow Height=""Auto"" />
						<GridRow Height=""1"" />
						<GridRow />
					</GridPanel.Rows>
					<GridPanel.Columns>
						<GridColumn />
					</GridPanel.Columns>

					<Text Name=""Title"" Text="" "" Font=""Verdana, 3world, style=Bold"" />
					<Edit Name=""Code"" Text="""" Font=""Verdana, 3world"" GridRow=""2"" TextChanged=""OnTextChanged"" />

				</GridPanel>
			</Border>";

		public OperationNode(Diagram diagram)
			: base(diagram)
		{
		}

		protected override void SaveToXml(XmlElement xmlElement, XmlPersistContext context)
		{
			base.SaveToXml(xmlElement, context);

			// Save type & code
			context.WriteInt((int)Type, "Type", xmlElement);
			context.WriteString(Code, "Code", xmlElement);
		}

		protected override void LoadFromXml(XmlElement xmlElement, XmlPersistContext context)
		{
			base.LoadFromXml(xmlElement, context);

			// Load type & code
			Type = (OperationType)context.ReadInt("Type", xmlElement);
			Code = context.ReadString("Code", xmlElement);
		}

		protected override ComponentBase LoadContent()
		{
			return XmlLoader.Load(ContentTemplate, this);
		}

		protected override void OnLoaded()
		{
			base.OnLoaded();

			code = FindComponent("Code") as EditComponent;
			title = FindComponent("Title") as TextComponent;
		}

		private void OnTextChanged(object sender, TextEventArgs e)
		{
			EnsureSize();
		}


		public OperationType Type
		{
			get { return type; }
			set
			{
				type = value;

				switch (type)
				{

					case OperationType.Unspecified:
						{
							AnchorPattern = null;
							title.Text = " ";
						}
						break;

					case OperationType.Choice:
						{
							title.Text = "Choice";
							AnchorPattern = new AnchorPattern(
								new AnchorPoint[]
								{
									new AnchorPoint(50, 0, true, false, MarkStyle.Custom),
									new AnchorPoint(50, 100, false, true, MarkStyle.Custom),
									new AnchorPoint(100, 50, false, true, MarkStyle.Custom),
								});
							AnchorPattern.Points[0].Tag = 0;
							AnchorPattern.Points[1].Tag = 1;
							AnchorPattern.Points[2].Tag = 2;
						}
						break;

					case OperationType.Operation:
						{
							title.Text = "Operation";
							AnchorPattern = new AnchorPattern(
								new AnchorPoint[]
								{
									new AnchorPoint(50, 0, true, false, MarkStyle.Custom),
									new AnchorPoint(50, 100, false, true, MarkStyle.Custom),
								});
							AnchorPattern.Points[0].Tag = 0;
							AnchorPattern.Points[1].Tag = 3;
						}
						break;

				}
			}
		}

		public string Code
		{
			get { return code.Text; }
			set { code.Text = value; }
		}


		private OperationType type;
		private TextComponent title;
		private EditComponent code;
	}

	public enum OperationType
	{
		Unspecified,
		Operation,
		Choice,
	}
}
