//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;
using System.Drawing;
using System.Drawing.Drawing2D;

using MindFusion.Diagramming.Components;


namespace MindFusion.Diagramming.WinForms.Samples.CS.Scripting
{
	public class ScriptedNode : CompositeNode
	{
		private static readonly string Template = @"
			<SimplePanel>
				<Border Name=""Selection"" Pen=""LightSteelBlue"" Visible=""False"" />
				<Border Name=""Highlight"" Pen=""Red"" Visible=""False"" />
				<Border Padding=""1"">
					<Border Name=""Frame"" Background=""#F0F0F0"" Pen=""Black"" />
				</Border>
			</SimplePanel>";

		public ScriptedNode(Diagram diagram)
			: base(diagram)
		{
			// Load the base content of the node
			this.root = XmlLoader.Load(Template, this);
			Components.Add(root);

			frame = FindComponent("Frame") as BorderComponent;
			selection = FindComponent("Selection") as BorderComponent;
			selection.Pen.Width = 1;
			selection.Pen.Alignment = PenAlignment.Inset;
			highlight = FindComponent("Highlight") as BorderComponent;
			highlight.Pen.Width = 1;
			highlight.Pen.Alignment = PenAlignment.Inset;

			ComponentBase content = LoadContent();
			if (content != null)
				frame.Content = content;

			HandlesStyle = HandlesStyle.Invisible;

			OnLoaded();
		}

		// do not use the built-in component serialization
		protected override bool SerializeComponents()
		{
			return false;
		}

		protected virtual ComponentBase LoadContent()
		{
			return null;
		}

		protected virtual void OnLoaded()
		{
		}

		protected void EnsureSize()
		{
			MindFusion.Drawing.IGraphics graphics = Parent.CreateMeasureGraphics();

			SizeF size = root.GetDesiredSize(new SizeF(float.PositiveInfinity, float.PositiveInfinity), graphics);

			RectangleF bounds = Bounds;
			if (bounds.Width < size.Width)
				bounds.Width = size.Width;
			if (bounds.Height < size.Height)
				bounds.Height = size.Height;
			Bounds = bounds;

			root.InvalidateLayout();

			graphics.Dispose();
		}

		public void OnSelected()
		{
			selection.Visible = true;
		}

		public void OnDeselected()
		{
			selection.Visible = false;
		}

		public void ShowHighlight()
		{
			highlight.Visible = true;
			if (Parent != null)
				Parent.Invalidate(Bounds);
		}

		public void HideHighlight()
		{
			highlight.Visible = false;
			if (Parent != null)
				Parent.Invalidate(Bounds);
		}


		public MindFusion.Drawing.Brush Background
		{
			get { return frame.Background; }
			set { frame.Background = value; }
		}


		private ComponentBase root;
		private BorderComponent frame;
		private BorderComponent selection;
		private BorderComponent highlight;
	}
}
