//
// Copyright (c) 2022, MindFusion LLC - Bulgaria.
//

using System;

using MindFusion.Diagramming.Components;
using MindFusion.Drawing;


namespace MindFusion.Diagramming.WinForms.Samples.CS.Scripting
{
	public class TerminatorNode : ScriptedNode
	{
		private static readonly string ContentTemplate = @"
			<Text Name=""Text"" Text="" "" />";

		public TerminatorNode(Diagram diagram)
			: base(diagram)
		{
			AllowIncomingLinks = false;
			AllowOutgoingLinks = false;
		}

		protected override void SaveToXml(System.Xml.XmlElement xmlElement, XmlPersistContext context)
		{
			base.SaveToXml(xmlElement, context);

			// Save type
			context.WriteInt((int)Type, "Type", xmlElement);
		}

		protected override void LoadFromXml(System.Xml.XmlElement xmlElement, XmlPersistContext context)
		{
			base.LoadFromXml(xmlElement, context);

			// Load type
			Type = (TerminatorNodeType)context.ReadInt("Type", xmlElement);
		}

		protected override ComponentBase LoadContent()
		{
			return XmlLoader.Load(ContentTemplate);
		}

		protected override void OnLoaded()
		{
			base.OnLoaded();

			text = FindComponent("Text") as TextComponent;
		}


		public TerminatorNodeType Type
		{
			get { return type; }
			set
			{
				type = value;
				if (type == TerminatorNodeType.Start)
				{
                    Background = new SolidBrush(System.Drawing.Color.FromArgb(102, 154, 204));
                    TextBrush = new SolidBrush(System.Drawing.Color.White);                    
                    text.Text = "Start";
					AllowIncomingLinks = false;
					AllowOutgoingLinks = true;
				}
				else if (type == TerminatorNodeType.End)
				{
                    Background = new SolidBrush(System.Drawing.Color.FromArgb(206, 0, 0));
                    TextBrush = new SolidBrush(System.Drawing.Color.White);
					text.Text = "End";
					AllowIncomingLinks = true;
					AllowOutgoingLinks = false;
				}
				else
				{
					Background = new SolidBrush(System.Drawing.Color.FromArgb(0xF0, 0xF0, 0xF0));
					text.Text = " ";
					AllowIncomingLinks = false;
					AllowOutgoingLinks = false;
				}
			}
		}


		private TerminatorNodeType type;
		private TextComponent text;
	}

	public enum TerminatorNodeType
	{
		Unspecified,
		Start,
		End,
	}
}
